
package eu.dnetlib.dhp.export;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import eu.dnetlib.dhp.schema.common.ModelConstants;
import eu.dnetlib.dhp.schema.oaf.*;
import eu.dnetlib.dhp.schema.oaf.utils.IdentifierFactory;
import eu.dnetlib.dhp.schema.oaf.utils.OafMapperUtils;
import scala.collection.immutable.Map;

import java.io.IOException;
import java.util.List;
import java.util.Optional;

public class ScholexplorerPatcher {

	private static final ObjectMapper mapper = new ObjectMapper();

	public static Publication patchPublication(String input) {
		return Optional
			.ofNullable(patchAtomicAction(input))
			.map(aa -> "Publication".equals(aa.getClazz().getSimpleName()) ? (Publication) aa.getPayload() : null)
			.orElse(null);
	}

	public static Dataset patchDataset(String input) {
		return Optional
			.ofNullable(patchAtomicAction(input))
			.map(aa -> "Dataset".equals(aa.getClazz().getSimpleName()) ? (Dataset) aa.getPayload() : null)
			.orElse(null);
	}

	public static Relation patchRelation(String input) {
		return Optional
			.ofNullable(patchAtomicAction(input))
			.map(aa -> "Relation".equals(aa.getClazz().getSimpleName()) ? (Relation) aa.getPayload() : null)
			.orElse(null);
	}

	public static AtomicActionSandro patchAtomicAction(String input) {
		try {
			AtomicActionSandro aa = parse(input);
			switch (aa.getClazz().getSimpleName()) {

				case "Publication":
					return patchPublication((Publication) aa.getPayload());
				case "Dataset":
					return patchDataset((Dataset) aa.getPayload());
				case "Relation":
					return patchRelation((Relation) aa.getPayload());
				default:
					return null;
			}
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}
	}

	public static AtomicActionSandro parse(String input) throws IOException {
		return mapper.readValue(input, AtomicActionSandro.class);
	}

	public static <T extends Oaf> AtomicActionSandro<T> asAtomicAction(T oaf) {
		return new AtomicActionSandro<T>((Class<T>) oaf.getClass(), oaf);
	}

	private static AtomicActionSandro patchRelation(Relation relation) {
		if ("IsCitedBy".equals(relation.getRelClass())) {
			relation.setRelClass(ModelConstants.IS_CITED_BY);
		}
		if ("IsReviewedBy".equals(relation.getRelClass())) {
			relation.setRelClass(ModelConstants.IS_REVIEWED_BY);
		}
		relation.getCollectedfrom().forEach(cf -> {
			final Map<String, KeyValue> map = DLIToOAF.collectedFromMap();
			if (map.contains(cf.getKey())) {
				KeyValue oa_cf = map.get(cf.getKey()).get();
				cf.setKey(oa_cf.getKey());
				cf.setValue(oa_cf.getValue());
			}
		});

		return new AtomicActionSandro<>(Relation.class, relation);
	}

	private static AtomicActionSandro patchDataset(Dataset dataset) {
		return new AtomicActionSandro<>(Dataset.class, patchEntity(dataset));
	}

	private static AtomicActionSandro<Publication> patchPublication(Publication publication) {
		return new AtomicActionSandro<>(Publication.class, patchEntity(publication));
	}

	private static <T extends Result> T patchEntity(T result) {
		final List<Instance> instance = Optional
			.ofNullable(result.getInstance())
			.orElseThrow(() -> new IllegalStateException("record with missing instance: " + result.getId()));

		instance.forEach(i -> {
			i.setPid(Lists.newArrayList());
			i.setAlternateIdentifier(result.getPid());
			i
				.setAccessright(
					OafMapperUtils
						.accessRight(
							ModelConstants.UNKNOWN, "Unknown", ModelConstants.DNET_ACCESS_MODES,
							ModelConstants.DNET_ACCESS_MODES));
		});

		result.setPid(Lists.newArrayList());

		Optional
			.ofNullable(result.getExternalReference())
			.orElse(Lists.newArrayList())
			.forEach(e -> e.setAlternateLabel(Lists.newArrayList()));

		String newId = IdentifierFactory.createDOIBoostIdentifier(result);
		if (newId != null && !newId.equals(result.getId())) {
			result.getOriginalId().add(result.getId());
			result.setId(newId);
		}

		return result;
	}

}
